#This code first describes how to perform within array normalization for 
#GCAT chips. The remainder of the code describes how to merge the within
#array normalized GCAT and Ontario data and then perform between array 
#normalization using median absolute deviation.

#Make sure the directory is set to the location of the targets file and
#GPR files for the GCAT chips.

#Read the targets file for the GCAT chips.
Gtargets<-read.csv(file.choose())

#Create a subset of the targets file that contains only the GPR files created 
#after scanning the top of the GCAT chips. Read those GPR files into R.
Gtop<-subset(Gtargets,Gtargets$Location %in% 'Top')
f<-function(x) as.numeric(x$Flags > -99)
RT<-read.maimages(Gtop, source="genepix.median", wt.fun=f)

#Create a subset of the targets file that contains only the GPR files created 
#after scanning the bottom of the GCAT chips. Read those GPR files into R.
Gbottom<-subset(Gtargets,Gtargets$Location %in% 'Bottom')
RB<-read.maimages(Gbottom, source="genepix.median", wt.fun=f)
 
#Bind the data from the top and bottom halves of the chip for each GCAT chip.
RGG<-rbind(RT,RB)

#Extract the column of values from the targets file that indicates which 
#chips were dye-swapped.
dw<-Gtop$Dyeswap

#Within Array Normalization
MAG<-normalizeWithinArrays(RGG,method="loess", bc.method="normexp")

#Create a list of the names of the genes on the GCAT chip.
M0<-tapply(MAG$M[,1],as.factor(MAG$genes$ID),mean)
gcatID<-rownames(M0)

#Designate which column of the targets file contains the name of each chip in
#the form <strain>_LogFC_t<time point>-<flask number>.
chips<-Gtop$Header

#Create a matrix MR to store the data after replicates have been averaged.
m0<-length(MAG$M[1,])
m1<-length(M0)
MR<-matrix(nrow=m1,ncol=m0)

#Set up a for loop to average replicates and store the data in matrix MR.
for(i in 1:length(chips)) {MR[,i]<-tapply(MAG$M[,i],as.factor(MAG$genes$ID),mean)}

#Create a new matrix MG to store the data.
MG<-matrix(nrow=m1,ncol=m0)

#Set up a for loop to multiply each row of the data matrix MR by the vector of
#values dw indicating the dye-swapped chips. Store the output to the matrix MG.
for(i in 1:m0) {MG[,i]<-dw[i]*MR[,i]}

#Convert matrix MG to a data frame. The row names of the data frame should
#be the names of the genes on the GCAT chps. The column names of the
#data frame should be the names of each chip that include the strain, time 
#point and flask number.
MG2<-as.data.frame.matrix(MG)
colnames(MG2)<-chips
rownames(MG2)<-gcatID

#In normalizing dSWI4 GCAT chip data collected by two classes of Biology 478 on 4/29/2014, 
#I noticed that the gene names in the GPR files included an "_01" at the end of the proper name, 
#which is unlike the GCAT chip data for other deletion strains that had been previously analyzed. 
#The following line of code was used to remove the "_01":
gcatIDtruncated<-strsplit(gcatID,"_01",fixed=TRUE)

#The row names of the data frame MG2 were then set using the following line of code:
rownames(MG2)<-gcatIDtruncated

#Merge the GCAT and Ontario chip within array normalized data. Set the
#row names of the new data frame to the names of the genes on both the
#Ontario and GCAT chips.
GO<-merge(MD4,MG2,by="row.names",all=T)
rownames(GO)<-GO$Row.names

#Get rid of the genes on the GCAT chips that are not on the Ontario chips. 
GO2<-subset(GO,rownames(GO) %in% ontID)

#Create a list to order the merged GCAT and Ontario chip data by strain, then 
#time point, and then flask.
MasterList<-rbind(Otargets[,c('Header','MasterList')],Gtop[,c('Header','MasterList')])
OrderedML<-MasterList[sort.list(MasterList$MasterList),]

#Rearrange the columns so that they are ordered by strain (wildtype then deletion strains
#in alphabetical order), timepoint, and then flask.
GO3<-GO2[,match(OrderedML$Header,colnames(GO2))]

#Write to a table.
write.table(GO3,"GCAT_and_Ontario_Within_Array_Normalization.csv",sep=",",col.names=NA,row.names=TRUE)

#Create a new matrix to store the data. It should have as many rows as there are genes 
#common to both the GCAT and Ontario chips and as many columns as there are GPR files
#that have been normalized.
g0<-length(GO3[1,])
g1<-length(GO3[,1])
MADM<-matrix(nrow=g1,ncol=g0)

#Scale each chips by its median absolute deviation.
for (i in 1:g0) {MADM[,i]<-GO3[,i]/mad(GO3[,i],na.rm=TRUE)}

#Convert matrix MADM to a data frame. The row names of the data frame should
#be the names of the genes common to the Ontario and GCAT chips. The column 
#names of the data frame should be the names of each chip that include the 
#strain, time point and flask number.
MAD<-as.data.frame.matrix(MADM)
rownames(MAD)<-rownames(GO3)
colnames(MAD)<-colnames(GO3)

#Write final to a table.
write.table(MAD,"GCAT_and_Ontario_Final_Normalized_Data.csv",sep=",",col.names=NA,row.names=TRUE)




